function [kappa]=asympfac(a,b,c,d,tol)
%
% by Bernd Fischer, version 25.08.93
%
% computes the asymptotic convergence factor
% relative to E = [a,b] u [c,d]
%
% desired accuracy of the result: tol
% 
% uses: sn_inv, JTheta_4
%

%%normalize E
   b_hat=(2*b-a-d)/(d-a);
   c_hat=(2*c-a-d)/(d-a);
   xi   =-(d+a)/(d-a);

%%compute k and rho
   k=sqrt(2*(c_hat-b_hat)/((1-b_hat)*(1+c_hat)));
   w_rho=sqrt((1-b_hat)/2);
   rho=-sn_inv(w_rho,k,tol);

%%complete elliptic integrals
   K=ellipk(1,k^2);
   K_prime=ellipk(1,1-k^2);

%%compute u_hat
   sn_rho=ellipj(rho,k^2);
   w_xi=-sqrt((xi-b_hat)/(xi+1))/(k*sn_rho);
   u_hat=-sn_inv(w_xi,k,tol);

%%compute kappa
   kappa=abs(jtheta_4((u_hat-rho),K,K_prime,100)/...
             jtheta_4((u_hat+rho),K,K_prime,100));

return;
