function [V,alpha,beta]=LANCZOS(A,r0,n,tol)
%
% by Bernd Fischer, version: 25.08.93
%
% computes an orthonormal basis of the Krylov subspace
%
%    K_n(A,r0) = span{v_1,v_2,...,v_n}
%
% and the entries of the Jacobi matrix J_n
%
% stops if beta_j < tol
%
% uses: matvec 
%

%%initialize
    j=1; N=length(r0); 
    v_old=zeros(N,1); v_hat=r0;
    beta(j)=norm(v_hat); 

%%iterate
   while (j < n+1) & (beta(j) > tol)

     v=v_hat/beta(j); Av=matvec(A,v); alpha(j)=v'*Av;
     v_hat=Av-alpha(j)*v-beta(j)*v_old; 
     j=j+1; beta(j)=norm(v_hat);
     v_old=v; V=[V,v];

   end; %while

return;

