function [s]=MPCI(t,x,y)
%
% by Bernd Fischer, version: 25.08.93
%
% computes and evaluates a 
% monotone piecewise cubic interpolant s(t)
%
% interpolation data:     (x,y)
% sample points:           t
%

%%initialize
   n=length(x);  d=zeros(n,1); I=1:n-1;
   m=length(t);  s=zeros(m,1);   
   h    =x(I+1)-x(I);
   delta=(y(I+1)-y(I))./h; I(n-1)=[];
   xi   =(h(I)+2*h(I+1))./(3*(h(I)+h(I+1)));

%%derivatives at x_j
   for j=2:n-1
       if delta(j-1)*delta(j) > 0
          d(j) =(delta(j-1) * delta(j))/ ...
                (xi(j-1)*delta(j)+(1-xi(j-1))*delta(j-1));
       end;
   end;

%%"not-a-knot" condition
   d(1)=h(1)*(3*delta(2)-(2*d(2)+d(3)))/h(2)+...
        3*delta(1)-2*d(2);
   d(n)=h(n-1)*(3*delta(n-1)-(2*d(n-1)+d(n-2)))/h(n-2)+...
        3*delta(n-1)-2*d(n-1);
   if d(1) < 0,          d(1)=0; end;
   if d(1) > 3*delta(1), d(1)=3*delta(1); end;
   if d(n) < 0,            d(n)=0; end;
   if d(n) > 3*delta(n-1), d(n)=3*delta(n-1); end;

%%evaluate spline
    I=find(t < x(1));
    if ~isempty(I)
        s(I)=y(1)*ones(length(I),1);
    end;
    for j=1:n-1
        I=find(t>=x(j) & t<x(j+1));
        if ~isempty(I)
           s(I)=((d(j)+d(j+1)-2*delta(j))/h(j)^2)*...
                (t(I)-x(j)).^3+((-2*d(j)-d(j+1)+...
                3*delta(j))/h(j))*(t(I)-x(j)).^2+...
                d(j)*(t(I)-x(j))+y(j);
        end;
    end;
    I=find(t >= x(n));
    if ~isempty(I)
        s(I)=y(n)*ones(length(I),1);
    end;
return;
