function [xCG,norm_rCG]=CG(A,f,x0,n_max,tol)
%
% by Bernd Fischer, version: 25.08.93
%
% computes the solution of Ax=f using the CG scheme.
% 
% startvector:                      x0
% maximal number of iterations:     n_max
% desired accuracy of the residual: tol
%
% uses: matvec
%

%%initialize
   n=1; N=length(x0);
   xCG=x0; rCG=f-feval(matvec,A,xCG); 
   rr=rCG'*rCG; rr_old=1;
   norm_rCG=sqrt(rr); norm_r0=norm_rCG; 
   w=zeros(N,1);

%%iterate
   while (n < n_max+1) &  (norm_rCG/norm_r0 > tol) 
      n=n+1;
       
      nu =rr/rr_old;
      w  =nu*w-rCG;
      Aw =matvec(A,w);
      eta=-rr/(w'*Aw);

   %%update
      xCG=xCG+eta*w;
      rCG=rCG-eta*Aw; 

      rr_old=rr; rr=rCG'*rCG;

   %%compute norm
      norm_rCG=sqrt(rr);

   end %while

return; 
