function [xCR,norm_rCR]=CR(A,f,x0,n_max,tol)
%
% by Bernd Fischer, version: 25.08.93
%
% computes the solution of Ax=f using the CR scheme.
% 
% startvector:                      x0
% maximal number of iterations:     n_max
% desired accuracy of the residual: tol
%
% uses: matvec
%

%%initialize
   n=1; N=length(x0);
   xCR=x0; rCR=f-matvec(A,xCR); 
   norm_rCR=norm(rCR); norm_r0=norm_rCR; 
   rAr=1;
   w=zeros(N,1); Aw=zeros(N,1);

%%iterate
   while (n < n_max+1) &  (norm_rCR/norm_r0 > tol) 
      n=n+1;

      rAr_old=rAr; 
      Ar  =matvec(A,rCR); rAr=rCR'*Ar; 
      nu  =rAr/rAr_old;
      w   =nu*w-rCR; Aw=nu*Aw-Ar;
      eta =-rAr/(Aw'*Aw);

   %%update
      xCR=xCR+eta*w;
      rCR=rCR-eta*Aw; 

   %%compute norm
      norm_rCR=norm(rCR);

   end %while

return; 
