function [xCRAIG,norm_rCRAIG]=CRAIG(A,f,x0,n_max,tol)
%
% by Bernd Fischer, version: 25.08.93
%
% computes the solution of Ax=f using the CRAIG scheme.
%
% startvector:                      x0
% maximal number of iterations:     n_max
% desired accuracy of the residual: tol
%
% uses: matvec
%

%%initialize
   n=1; N=length(x0); 
   v=zeros(N,1); 
   u=f-matvec(A,x0); beta=norm(u); u=u/beta;
   xi=-1;
   xCRAIG=x0; norm_rCRAIG=beta; norm_r0=beta;
   
   while (n < n_max+1) & (norm_rCRAIG/norm_r0 > tol) 
      n=n+1;

   %%Bidiagonalization
      Au=matvec(A,u);
      v=Au-beta*v; alpha=norm(v); v=v/alpha;
      Av=matvec(A,v); beta_old=beta;
      u=Av-alpha*u; beta=norm(u); u=u/beta;
   
   %%update
      xi=-xi*beta_old/alpha;
      xCRAIG=xCRAIG+xi*v; 

   %%compute norm
      norm_rCRAIG=beta*abs(xi);

    end %while

return;