function [xLSQR,norm_rLSQR]=LSQR(A,f,x0,n_max,tol)
%
% by Bernd Fischer, version: 25.08.93
%
% computes the solution of Ax=f using the LSQR scheme.
%
% startvector:                      x0
% maximal number of iterations:     n_max
% desired accuracy of the residual: tol
%
% uses: matvec
%

%%initialize
   n=1; N=length(x0); 
   v=zeros(N,1); 
   u=f-matvec(A,x0); beta=norm(u); u=u/beta;
   c=1; s=0; 
   w=zeros(N,1); eta=beta;
   xLSQR=x0; norm_rLSQR=beta; norm_r0=beta;
   
   while (n < n_max+1) & (norm_rLSQR/norm_r0 > tol) 
      n=n+1;

   %%Bidiagonalization
      Au=matvec(A,u);
      v=Au-beta*v; alpha=norm(v); v=v/alpha;
      Av=matvec(A,v); 
      u=Av-alpha*u; beta=norm(u); u=u/beta;

   %%QR factorization
      r1_hat=c*alpha; 
      r1    =sqrt(r1_hat^2+beta^2);
      r2    =s*alpha;
     
   %%Givens rotation
      c=r1_hat/r1;
      s=beta/r1; 
   
   %%update
      w=(v-r2*w)/r1; 
      xLSQR=xLSQR+c*eta*w; 
      eta=-s*eta;

   %%compute norm
      norm_rLSQR=norm_rLSQR*abs(s);
      
    end %while

return;
