function [xMCR,norm_rMCR]=MCR(A,f,x0,n_max,tol)
%
% by Bernd Fischer, version: 25.08.93
%
% computes the solution of Ax=f using the MCR scheme.
%
% startvector:                      x0
% maximal number of iterations:     n_max
% desired accuracy of the residual: tol
%
% uses: matvec
%

%%initialize
   n=1; N=length(x0); 
   v=f-matvec(A,x0); v_old=zeros(N,1); beta=0;
   Av=matvec(A,v);  Av_old=zeros(N,1);
   xMCR=x0; rMCR=v; 
   norm_rMCR=norm(v); norm_r0=norm_rMCR;
   
   while (n<n_max+1) & (norm_rMCR/norm_r0>tol) 
      n=n+1;
      AAv=matvec(A,Av);  AvAv=Av'*Av;
      rho=rMCR'*Av/AvAv;
   
   %%update
      xMCR=xMCR+rho*v; 
      rMCR=rMCR-rho*Av;  

   %%compute norm
      norm_rMCR=norm(rMCR);

   %%monic polynomial
      alpha=Av'*AAv/AvAv;
      if n > 2, beta=AvAv/AvAv_old; end;

      v_oold=v_old; v_old=v;  
      v =Av-alpha*v_old-beta*v_oold;    
      Av_oold=Av_old; Av_old=Av; 
      Av=AAv-alpha*Av_old-beta*Av_oold; 
      AvAv_old=AvAv;

    end %while

return;
