function [xMR,norm_rMR,xOR,norm_rOR]=MINRES(A,f,x0,n_max,tol)
%
% by Bernd Fischer, version: 25.08.93
%
% computes the solution of Ax=f using the MINRES scheme.
%
% startvector:                      x0
% maximal number of iterations:     n_max
% desired accuracy of the residual: tol
%
% uses: matvec
%

%%initialize
   n=1; N=length(x0); 
   v=zeros(N,1); v_hat=f-matvec(A,x0); beta=norm(v_hat); 
   c=1; c_old=1; s_old=0; s=0; 
   w=zeros(N,1); w_old=w; eta=beta;
   xMR=x0; norm_rMR=beta; norm_r0=beta;
   
   while (n < n_max+1) & (norm_rMR/norm_r0 > tol) 
      n=n+1;

   %%Lanczos
      v_old=v;
      v=v_hat/beta; Av=matvec(A,v); alpha=v'*Av;
      v_hat=Av-alpha*v-beta*v_old;
      beta_old=beta; beta=norm(v_hat);

   %%QR factorization
      c_oold=c_old; c_old=c; s_oold=s_old; s_old=s;

      r1_hat=c_old*alpha-c_oold*s_old*beta_old;
      r1    =sqrt(r1_hat^2+beta^2);
      r2    =s_old*alpha+c_oold*c_old*beta_old;
      r3    =s_oold*beta_old;

   %%Givens rotation
      c=r1_hat/r1;
      s=beta/r1; 
   
   %%update
      w_oold=w_old; w_old=w; 
      w=(v-r3*w_oold-r2*w_old)/r1; 
      xMR=xMR+c*eta*w; norm_rMR=norm_rMR*abs(s);
      eta=-s*eta;

    end %while

    if abs(c) > eps
       xOR     =xMR-s*eta*w/c;
       norm_rOR=norm_rMR/abs(c);
    end;

return;
