function [xOR,norm_rOR,xME,norm_rME,xMR,norm_rMR]=...
  SYMMLQ(A,f,x0,n_max,tol)
%
% by Bernd Fischer, version: 25.08.93
%
% computes the solution of Ax=f using the SYMMLQ scheme.
%
% startvector:                      x0
% maximal number of iterations:     n_max
% desired accuracy of the residual: tol
%
% uses: matvec
%

%%initialize
   n=1; N=length(x0); 
   v_hat=f-matvec(A,x0); beta=norm(v_hat); 
   v=v_hat/beta; v_old=zeros(N,1);
   c=1; c_old=1; s_old=0; s=0; r1=1;
   xME=x0; xMR=x0; w=zeros(N,1); 
   norm_rMR=beta; norm_r0=beta;  eta=-1; eta_old=0;

   while (n < n_max+1) & (norm_rMR/norm_r0 > tol) 
      n=n+1;
   %%Lanczos    
      Av=matvec(A,v); alpha=v'*Av;
      v_hat=Av-alpha*v-beta*v_old;
      beta_old=beta; beta=norm(v_hat); v_old=v;
   %%QR factorization
      c_oold=c_old; c_old=c; s_oold=s_old; s_old=s; r1_old=r1;
      r1_hat=c_old*alpha-c_oold*s_old*beta_old;
      r1    =sqrt(r1_hat^2+beta^2);
      r2    =s_old*alpha+c_oold*c_old*beta_old;
      r3    =s_oold*beta_old;
   %%Givens rotation
      c=r1_hat/r1;
      s=beta/r1; 
   %%update
      w=-s_old*w+c_old*v; v=v_hat/beta;  
      eta_oold=eta_old; eta_old=eta;
      eta=-(r3*eta_oold+r2*eta_old)/r1;
     
      xMR     =s^2*xMR+c^2*xME+c*eta*w;
      norm_rMR=norm_rMR*abs(s);
      xME     =xME+eta*(c*w+s*v); %wME =c*w+s*v;
      norm_rME=sqrt((r1_old*eta_old)^2+(r3*eta_oold)^2);   
    end %while

    if abs(c) > eps
       xOR     =xME+eta*w/c;
       norm_rOR=beta*abs(s_old*eta_old+c_old*eta/c);
    end;

return;
